import requests

from requests.adapters import HTTPAdapter


class HTTPtimeoutAdapter(HTTPAdapter):
    """ Adds timeout to requests HTTP adapter """

    def __init__(self, timeout: int = 5, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.timeout = timeout

    def send(self, *args, **kwargs):
        if kwargs.get('timeout') is None:
            kwargs['timeout'] = self.timeout
        return super(HTTPtimeoutAdapter, self).send(*args, **kwargs)


class SunshineBase:
    _OBJECTS_ROOT = 'api/sunshine/objects'

    _RELATIONSHIPS_ROOT = 'api/sunshine/relationships'

    _PROFILES_ROOT = 'api/v2/user_profiles'

    _EVENTS_ROOT = 'api/v2/user_profiles'

    _USERS_ROOT = 'api/v2/users'

    _HEADERS = {
        'Content-type': 'application/json',
        'Accept': 'application/json'
    }

    def __init__(self, subdomain: str, email: str, key: str, global_timeout: int = 5):
        """
        Creates and manages objects on zendesk sunshine

        :param subdomain: Your zendesk subdomain
        :param email: Sunshine user email ID. Used for token based authentication
        :param key: Sunshine token.
        :param global_timeout: Request timeout in seconds for every request within the session
        """

        self.subdomain = subdomain
        self._objects_base_url = f'https://{self.subdomain}.zendesk.com/{self._OBJECTS_ROOT}'
        self._relationships_base_url = f'https://{self.subdomain}.zendesk.com/{self._RELATIONSHIPS_ROOT}'
        self._profiles_base_url = f'https://{self.subdomain}.zendesk.com/{self._PROFILES_ROOT}'
        self._events_base_url = f'https://{self.subdomain}.zendesk.com/{self._EVENTS_ROOT}'
        self._users_base_url = f'https://{self.subdomain}.zendesk.com/{self._USERS_ROOT}'

        self._session = requests.Session()

        self._adapter = HTTPtimeoutAdapter(global_timeout)
        self._session.mount('http://', self._adapter)
        self._session.mount('https://', self._adapter)

        self._session.headers.update(self._HEADERS)

        self._session.auth = (f'{email}/token', f'{key}')

    def __repr__(self):
        return f'{self.__class__.__name__}(subdomain="{self.subdomain}")'

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._session.close()

    def close_session(self):
        """ Closes requests session """

        self._session.close()



from hns_sunshine_api.sunshine.base import SunshineBase
from requests import Response


class SunshineEvents(SunshineBase):
    """ Sunshine Profiles class """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return f'{self.__class__.__name__}(subdomain="{self.subdomain}")'

    def get_by_profile(self, identifier: str) -> Response:
        """
        Gets Sunshine events linked to a Sunshine profile by profile identifier
        :param identifier: Profile identifier. https://develop.zendesk.com/hc/en-us/articles/360043415094
        :return: Events
        """

        return self._session.get(f'{self._events_base_url}/events', params={'identifier': identifier})

    def get_by_id(self, profile_id: str):
        """
        Gets Sunshine events linked to a Sunshine profile by profile ID
        :param profile_id: Sunshine profile ID
        :return: Events
        """

        return self._session.get(f'{self._events_base_url}/{profile_id}/events')

    def track_against_profile(self, event_data: dict) -> Response:
        """
        Tracks events against a Sunshine profile using Profile Identifier. Identifiers are in event_data
        :param event_data: Event data. https://develop.zendesk.com/hc/en-us/articles/360044045834-Anatomy-of-a-Sunshine-event
        :return: Events
        """

        return self._session.post(f'{self._events_base_url}/events', json=event_data)

    def track_against_profile_id(self, profile_id: str, event_data: dict) -> Response:
        """
        Tracks events against a Sunshine profile using Profile ID
        :param profile_id: Sunshine profile ID
        :param event_data: Event data. https://develop.zendesk.com/hc/en-us/articles/360044045834-Anatomy-of-a-Sunshine-event
        :return: Events
        """

        return self._session.post(f'{self._events_base_url}/{profile_id}/events', json=event_data)




"""
Zendesk sunshine API
API guide: https://developer.zendesk.com/rest_api/docs/sunshine/introduction
"""
from hns_sunshine_api.sunshine.object_records import SunshineObjectRecords
from hns_sunshine_api.sunshine.relationship_records import SunshineRelationshipRecords
from hns_sunshine_api.sunshine.profiles import SunshineProfiles
from hns_sunshine_api.sunshine.events import SunshineEvents


class Sunshine:

    def __init__(self, subdomain: str, email: str, key: str, global_timeout: int = 5):
        """
        Creates and manages objects on zendesk sunshine

        Instantiating creates a requests session. You can call `close` at the end.

        Usage:

        Simple use:

            sunshine = Sunshine(subdomain='mydomain', email='test@example.com', key='test_key')
            objects = sunshine.object_records().list(object_type='sometype')
            relation = sunshine.relationships_records().show(relationship_id='someid')
            sunshine.close_session()

        Using context manager:

            with Sunshine(subdomain='mydomain', email='test@example.com', key='test_key') as sunshine:
                objects = sunshine.object_records().list(object_type='sometype')
                relation = sunshine.relationships_records().show(relationship_id='someid')

        Zendesk API docs: https://developer.zendesk.com/rest_api/docs/sunshine/introduction

        :param subdomain: Your zendesk subdomain
        :param email: Sunshine user email ID. Used for token based authentication
        :param key: Sunshine token.
        :param global_timeout: Request timeout in seconds for every request within the session
        """

        self.subdomain = subdomain
        self._object_record = SunshineObjectRecords(
            subdomain=subdomain, email=email, key=key, global_timeout=global_timeout
        )
        self._relationship_record = SunshineRelationshipRecords(
            subdomain=subdomain, email=email, key=key, global_timeout=global_timeout
        )
        self._profiles = SunshineProfiles(
            subdomain=subdomain, email=email, key=key, global_timeout=global_timeout
        )
        self._events = SunshineEvents(
            subdomain=subdomain, email=email, key=key, global_timeout=global_timeout
        )

    def __repr__(self):
        return f'{self.__class__.__name__}(subdomain="{self.subdomain}")'

    def object_records(self):
        return self._object_record

    def relationships_records(self):
        return self._relationship_record

    def object_types(self):
        raise NotImplementedError

    def relationship_types(self):
        raise NotImplementedError

    def events(self):
        return self._events

    def profiles(self):
        return self._profiles

